//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


// SharedFilesCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "SharedFilesCtrl.h"
#include "otherfunctions.h"


// CSharedFilesCtrl

IMPLEMENT_DYNAMIC(CSharedFilesCtrl, CListCtrl)
CSharedFilesCtrl::CSharedFilesCtrl(){
	
}

CSharedFilesCtrl::~CSharedFilesCtrl(){
}

void CSharedFilesCtrl::Init(){
	SetExtendedStyle(LVS_EX_FULLROWSELECT);
	InsertColumn(0,"Filename",LVCFMT_LEFT,250,0);
	InsertColumn(1,"Size",LVCFMT_LEFT,100,1);
	InsertColumn(2,"Type",LVCFMT_LEFT,50,2);
	InsertColumn(3,"Priority",LVCFMT_LEFT,70,3);
	InsertColumn(4,"Permission",LVCFMT_LEFT,100,4);
	InsertColumn(5,"FileID",LVCFMT_LEFT,220,5);

	m_SharedFilesMenu.CreatePopupMenu();
	m_SharedFilesMenu.AddMenuTitle("Shared Files");
	
	// add priority switcher
	m_PrioMenu.CreateMenu();
	m_PrioMenu.AppendMenu(MF_STRING,MP_PRIOVERYLOW,"Very Low");
	m_PrioMenu.AppendMenu(MF_STRING,MP_PRIOLOW,"Low");
	m_PrioMenu.AppendMenu(MF_STRING,MP_PRIONORMAL,"Normal");
	m_PrioMenu.AppendMenu(MF_STRING,MP_PRIOHIGH, "High");
	m_PrioMenu.AppendMenu(MF_STRING,MP_PRIOVERYHIGH, "Release");

	// add permission switcher
	m_PermMenu.CreateMenu();
	m_PermMenu.AppendMenu(MF_STRING,MP_PERMNONE,	"Locked");
	m_PermMenu.AppendMenu(MF_STRING,MP_PERMFRIENDS,	"Friends only");
	m_PermMenu.AppendMenu(MF_STRING,MP_PERMALL,		"Public");


	// todo enable when it works
	//m_SharedFilesMenu.AppendMenu(MF_STRING|MF_POPUP,(UINT_PTR)m_PrioMenu.m_hMenu, (LPCTSTR)"Priority");
	//m_SharedFilesMenu.AppendMenu(MF_STRING|MF_POPUP,(UINT_PTR)m_PermMenu.m_hMenu, (LPCTSTR)"Permissions");
	//m_SharedFilesMenu.AppendMenu(MF_STRING|MF_SEPARATOR);
	m_SharedFilesMenu.AppendMenu(MF_STRING,MP_OPEN, (LPCTSTR)"Open file");
	m_SharedFilesMenu.AppendMenu(MF_STRING,MP_GETED2KLINK, (LPCTSTR)"Copy ED2k link to clipboard");
	m_SharedFilesMenu.AppendMenu(MF_STRING,MP_GETHTMLED2KLINK, (LPCTSTR)"Copy ED2k link to clipboard (HTML)");
}

void CSharedFilesCtrl::ShowFileList(CSharedFileList* in_sflist){
	DeleteAllItems();
	sflist = in_sflist;
	for (POSITION pos = sflist->list.GetHeadPosition();pos != 0;sflist->list.GetNext(pos))
		ShowFile(sflist->list.GetAt(pos));
	
}

void CSharedFilesCtrl::ShowFile(CKnownFile* file){
	uint32 itemnr = GetItemCount();
	InsertItem(LVIF_TEXT|LVIF_PARAM,itemnr,file->GetFileName(),0,0,0,(LPARAM)file);
	char buffer[50];
	CastItoXBytes(file->GetFileSize(),buffer);
	SetItemText(itemnr,1,buffer);
	if (file->GetPriority() == PR_VERYLOW)
		SetItemText(itemnr,3,"Lowest");
	else if (file->GetPriority() == PR_LOW)
		SetItemText(itemnr,3,"Low");
	else if (file->GetPriority() == PR_NORMAL)
		SetItemText(itemnr,3,"Normal");
	else if (file->GetPriority() == PR_HIGH)
		SetItemText(itemnr,3,"High");
	else
		SetItemText(itemnr,3,"Highest");

	/*if (file->GetPermissions() == PERM_NOONE)
		SetItemText(itemnr,4,"Hidden");
	else if (file->GetPermissions() == PERM_FRIENDS)
		SetItemText(itemnr,4,"Friends Only");
	else
		SetItemText(itemnr,4,"Public Shared");*/
	SetItemText(itemnr,4,"Public Shared");

	buffer[0] = 0;
	for (uint16 i = 0;i != 16;i++) // hmm I wonder if there is a standard function for this
		sprintf(buffer,"%s%02X",buffer,file->GetFileHash()[i]);
	SetItemText(itemnr,5,buffer);
}

BEGIN_MESSAGE_MAP(CSharedFilesCtrl, CListCtrl)
	ON_WM_CONTEXTMENU()
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnClick)
END_MESSAGE_MAP()



// CSharedFilesCtrl message handlers
void CSharedFilesCtrl::OnContextMenu(CWnd* pWnd, CPoint point)
{
	m_SharedFilesMenu.TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON,point.x,point.y,this);
}



BOOL CSharedFilesCtrl::OnCommand(WPARAM wParam, LPARAM lParam)
{
	int iSel = GetSelectionMark();
	if (iSel != (-1)){
		CKnownFile* file = (CKnownFile*)GetItemData(iSel);
		switch (wParam){
			case MP_GETED2KLINK:
				{
					theApp.CopyTextToClipboard(theApp.CreateED2kLink(file));
					break;
				}
			case MP_GETHTMLED2KLINK:
				{
					theApp.CopyTextToClipboard(theApp.CreateHTMLED2kLink(file));
					break;
				}
			case MP_OPEN:
				{
					char* buffer = new char[250];
					sprintf(buffer,"%s\\%s",file->GetPath(),file->GetFileName());
					theApp.emuledlg->AddLogLine( false, "%s\\%s",file->GetPath(),file->GetFileName());
					ShellOpenFile(buffer);
					delete buffer;
					break; 

				}
			case MP_PRIOVERYLOW:
				{	file->SetPriority(PR_VERYLOW);SetItemText(iSel,3,"Lowest");break;	}
			case MP_PRIOLOW:
				{	file->SetPriority(PR_LOW);SetItemText(iSel,3,"Low");break;	}
			case MP_PRIONORMAL:
				{	file->SetPriority(PR_NORMAL);SetItemText(iSel,3,"Normal");break;	}
			case MP_PRIOHIGH:
				{	file->SetPriority(PR_HIGH);SetItemText(iSel,3,"High");break;	}
			case MP_PRIOVERYHIGH:
				{	file->SetPriority(PR_VERYHIGH);SetItemText(iSel,3,"Highest");break;	}

			case MP_PERMNONE:
				{
					if (((CPartFile*)file)->IsPartFile())
						AfxMessageBox("You cannot change permissions while a file is still downloading!");
					else
					{	
						file->SetPermissions(PERM_NOONE);
						SetItemText(iSel,4,"Hidden");
					}
					break;
				}
			case MP_PERMFRIENDS:
				{
					if (((CPartFile*)file)->IsPartFile())
						AfxMessageBox("You cannot change permissions while a file is still downloading!");
					else
					{	
						file->SetPermissions(PERM_FRIENDS);
						SetItemText(iSel,4,"Friends Only");	
					}
					break;
				}
			case MP_PERMALL:
				{	file->SetPermissions(PERM_ALL);SetItemText(iSel,4,"Public Shared");break;	}
		}
	}
	return true;
}


void CSharedFilesCtrl::OnColumnClick( NMHDR* pNMHDR, LRESULT* pResult){
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	// if it's a second click on the same column then reverse the sort order,
	// otherwise sort the new column in ascending order.
	asc_sort[pNMListView->iSubItem] = !asc_sort[pNMListView->iSubItem];
	SortItems(&SortProc,pNMListView->iSubItem+ ((asc_sort[pNMListView->iSubItem])? 0:10));
	*pResult = 0;
}

int CSharedFilesCtrl::SortProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort){
	CKnownFile* item1 = (CKnownFile*)lParam1;
	CKnownFile* item2 = (CKnownFile*)lParam2;	
	switch(lParamSort){
		case 0: //filename asc
			return strcmpi(item1->GetFileName(),item2->GetFileName());
		case 10: //filename desc
			return strcmpi(item2->GetFileName(),item1->GetFileName());

		case 1: //filesize asc
			return item2->GetFileSize()-item1->GetFileSize();
		case 11: //filesize desc
			return item1->GetFileSize()-item2->GetFileSize();
/*
		case 2: //filetype asc
			return strcmpi( GetFiletypeByName( (CString)item1->GetFileName()),GetFiletypeByName( (CString)item2->GetFileType() ));
		case 12: //filetype desc
			return strcmpi(GetFiletypeByName((CString)item2->GetFileName()),GetFiletypeByName((CString)item1->GetFileType()));
*/
		case 3: //prio asc
			return item2->GetPriority()-item1->GetPriority();
		case 13: //prio desc
			return item1->GetPriority()-item2->GetPriority();

		case 4: //permission asc
			return item2->GetPermissions()-item1->GetPermissions();
		case 14: //permission desc
			return item1->GetPermissions()-item2->GetPermissions();

		case 5: //fileID asc
			return strcmpi((CString)item1->GetFileHash(),(CString)item2->GetFileHash());
		case 15: //fileID desc
			return strcmpi((CString)item2->GetFileHash(),(CString)item1->GetFileHash());

		default: 
			return 0;
	}
}